/**
 * DevExtreme (ui/file_manager/ui.file_manager.file_uploader.js)
 * Version: 19.2.1 (build 19246-0308)
 * Build date: Tue Sep 03 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _get = function get(object, property, receiver) {
    if (null === object) {
        object = Function.prototype
    }
    var desc = Object.getOwnPropertyDescriptor(object, property);
    if (void 0 === desc) {
        var parent = Object.getPrototypeOf(object);
        if (null === parent) {
            return
        } else {
            return get(parent, property, receiver)
        }
    } else {
        if ("value" in desc) {
            return desc.value
        } else {
            var getter = desc.get;
            if (void 0 === getter) {
                return
            }
            return getter.call(receiver)
        }
    }
};
var _renderer = require("../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _extend = require("../../core/utils/extend");
var _events_engine = require("../../events/core/events_engine");
var _events_engine2 = _interopRequireDefault(_events_engine);
var _deferred = require("../../core/utils/deferred");
var _ui = require("../widget/ui.widget");
var _ui2 = _interopRequireDefault(_ui);
var _guid = require("../../core/guid");
var _guid2 = _interopRequireDefault(_guid);
var _uiFile_manager = require("./ui.file_manager.common");
var _uiFile_manager2 = _interopRequireDefault(_uiFile_manager);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var FILE_MANAGER_FILE_UPLOADER_CLASS = "dx-filemanager-fileuploader";
var FILE_MANAGER_FILE_UPLOADER_FILE_INPUT_CLASS = FILE_MANAGER_FILE_UPLOADER_CLASS + "-fileinput";
var FileManagerFileUploader = function(_Widget) {
    _inherits(FileManagerFileUploader, _Widget);

    function FileManagerFileUploader() {
        _classCallCheck(this, FileManagerFileUploader);
        return _possibleConstructorReturn(this, (FileManagerFileUploader.__proto__ || Object.getPrototypeOf(FileManagerFileUploader)).apply(this, arguments))
    }
    _createClass(FileManagerFileUploader, [{
        key: "_initMarkup",
        value: function() {
            this._initActions();
            this._sessionMap = {};
            this.$element().addClass(FILE_MANAGER_FILE_UPLOADER_CLASS);
            this._renderFileInput();
            _get(FileManagerFileUploader.prototype.__proto__ || Object.getPrototypeOf(FileManagerFileUploader.prototype), "_initMarkup", this).call(this)
        }
    }, {
        key: "_renderFileInput",
        value: function() {
            this._$fileInput = (0, _renderer2.default)("<input>").attr("type", "file").prop({
                multiple: "multiple",
                tabIndex: -1
            }).addClass(FILE_MANAGER_FILE_UPLOADER_FILE_INPUT_CLASS);
            _events_engine2.default.on(this._$fileInput, "change", this._onFileInputChange.bind(this));
            _events_engine2.default.on(this._$fileInput, "click", function(e) {
                e.stopPropagation();
                return true
            });
            this.$element().append(this._$fileInput)
        }
    }, {
        key: "_onFileInputChange",
        value: function() {
            var files = this._$fileInput.prop("files");
            if (0 === files.length) {
                return
            }
            _events_engine2.default.off(this._$fileInput, "change");
            _events_engine2.default.off(this._$fileInput, "click");
            var $fileInput = this._$fileInput;
            this._uploadFiles(files).always(function() {
                setTimeout(function() {
                    $fileInput.remove()
                })
            });
            this._renderFileInput()
        }
    }, {
        key: "_uploadFiles",
        value: function(files) {
            var _this2 = this;
            if (0 === files.length) {
                return
            }
            var sessionId = (new _guid2.default).toString();
            var controllerGetter = this.option("getController");
            var session = new FileManagerUploadSession({
                id: sessionId,
                controller: controllerGetter(),
                onProgress: function(e) {
                    e.sessionId = sessionId;
                    _this2._raiseUploadProgress(e)
                },
                onError: function(reason) {
                    return _this2._raiseOnErrorOccurred(reason)
                }
            });
            this._sessionMap[sessionId] = session;
            var deferreds = session.uploadFiles(files);
            var sessionInfo = {
                sessionId: sessionId,
                deferreds: deferreds,
                files: files
            };
            this._raiseUploadSessionStarted(sessionInfo);
            return (0, _uiFile_manager2.default)(deferreds).always(function() {
                setTimeout(function() {
                    delete _this2._sessionMap[sessionId]
                })
            })
        }
    }, {
        key: "_getSession",
        value: function(id) {
            return this._sessionMap[id]
        }
    }, {
        key: "tryUpload",
        value: function() {
            this._$fileInput.click()
        }
    }, {
        key: "cancelUpload",
        value: function(sessionId) {
            var session = this._getSession(sessionId);
            if (session) {
                session.cancelUpload()
            }
        }
    }, {
        key: "cancelFileUpload",
        value: function(sessionId, fileIndex) {
            var session = this._getSession(sessionId);
            if (session) {
                session.cancelFileUpload(fileIndex)
            }
        }
    }, {
        key: "_onFilesUploaded",
        value: function() {
            this._actions.onFilesUploaded()
        }
    }, {
        key: "_raiseOnErrorOccurred",
        value: function(args) {
            this._actions.onErrorOccurred({
                info: args
            })
        }
    }, {
        key: "_raiseUploadSessionStarted",
        value: function(sessionInfo) {
            this._actions.onUploadSessionStarted({
                sessionInfo: sessionInfo
            })
        }
    }, {
        key: "_raiseUploadProgress",
        value: function(args) {
            this._actions.onUploadProgress(args)
        }
    }, {
        key: "_initActions",
        value: function() {
            this._actions = {
                onUploadSessionStarted: this._createActionByOption("onUploadSessionStarted"),
                onUploadProgress: this._createActionByOption("onUploadProgress"),
                onFilesUploaded: this._createActionByOption("onFilesUploaded"),
                onErrorOccurred: this._createActionByOption("onErrorOccurred")
            }
        }
    }, {
        key: "_getDefaultOptions",
        value: function() {
            return (0, _extend.extend)(_get(FileManagerFileUploader.prototype.__proto__ || Object.getPrototypeOf(FileManagerFileUploader.prototype), "_getDefaultOptions", this).call(this), {
                getController: null,
                onUploadSessionStarted: null,
                onUploadProgress: null,
                onFilesUploaded: null,
                onErrorOccurred: null
            })
        }
    }, {
        key: "_optionChanged",
        value: function(args) {
            var name = args.name;
            switch (name) {
                case "getController":
                    this.repaint();
                    break;
                case "onFilesUploaded":
                case "onErrorOccurred":
                case "onUploadSessionStarted":
                case "onUploadProgress":
                    this._actions[name] = this._createActionByOption(name);
                    break;
                default:
                    _get(FileManagerFileUploader.prototype.__proto__ || Object.getPrototypeOf(FileManagerFileUploader.prototype), "_optionChanged", this).call(this, args)
            }
        }
    }]);
    return FileManagerFileUploader
}(_ui2.default);
var FileManagerUploadSession = function() {
    function FileManagerUploadSession(options) {
        _classCallCheck(this, FileManagerUploadSession);
        this._controller = options.controller;
        this._onProgressHandler = options.onProgress;
        this._onErrorHandler = options.onError;
        this._canceled = false;
        this._cancellationState = {}
    }
    _createClass(FileManagerUploadSession, [{
        key: "uploadFiles",
        value: function(files) {
            var progressInfo = {
                uploadedBytesCount: 0,
                totalBytesCount: 0
            };
            for (var j = 0; j < files.length; j++) {
                progressInfo.totalBytesCount += files[j].size
            }
            var result = [];
            for (var i = 0; i < files.length; i++) {
                var deferred = this._uploadFile(files[i], i, progressInfo);
                result.push(deferred)
            }
            return result
        }
    }, {
        key: "cancelUpload",
        value: function() {
            this._canceled = true
        }
    }, {
        key: "cancelFileUpload",
        value: function(fileIndex) {
            this._cancellationState[fileIndex] = true
        }
    }, {
        key: "_uploadFile",
        value: function(file, fileIndex, progressInfo) {
            var _this3 = this;
            var state = this._createUploadingState(file, fileIndex);
            return this._controller.initiateUpload(state).then(function() {
                return _this3._uploadChunks(state, progressInfo)
            }).then(function() {
                return _this3._finalizeUpload(state)
            }, function(reason) {
                if (reason && reason.canceled) {
                    return _this3._abortUpload(state)
                } else {
                    return (new _deferred.Deferred).reject(reason).promise()
                }
            })
        }
    }, {
        key: "_uploadChunks",
        value: function(state, progressInfo) {
            var _this4 = this;
            if (this._canceled || this._cancellationState[state.fileIndex]) {
                var reason = this._createResultInfo(state.file.name, false, true);
                return (new _deferred.Deferred).reject(reason).promise()
            }
            var chunk = this._getChunk(state);
            if (!chunk) {
                return (new _deferred.Deferred).resolve().promise()
            }
            return this._controller.uploadChunk(state, chunk).done(function() {
                state.uploadedBytesCount += chunk.size;
                state.uploadedChunksCount++;
                progressInfo.uploadedBytesCount += chunk.size;
                _this4._raiseOnProgress(progressInfo, state)
            }).then(function() {
                return _this4._uploadChunks(state, progressInfo)
            })
        }
    }, {
        key: "_getChunk",
        value: function(state) {
            var bytesLeft = state.file.size - state.uploadedBytesCount;
            if (0 === bytesLeft) {
                return null
            }
            var chunkSize = Math.min(bytesLeft, this._controller.chunkSize);
            var blob = state.file.slice(state.uploadedBytesCount, state.uploadedBytesCount + chunkSize);
            return {
                index: state.uploadedChunksCount,
                size: chunkSize,
                blob: blob
            }
        }
    }, {
        key: "_finalizeUpload",
        value: function(state) {
            var _this5 = this;
            return this._controller.finalizeUpload(state).then(function() {
                return _this5._createResultInfo(state.file.name, true)
            })
        }
    }, {
        key: "_abortUpload",
        value: function(state) {
            var _this6 = this;
            return this._controller.abortUpload(state).then(function() {
                return _this6._createResultInfo(state.file.name, false, true)
            })
        }
    }, {
        key: "_handleError",
        value: function(error, file) {
            var result = this._createResultInfo(file.name, false, false, error);
            this._onErrorHandler(result);
            return result
        }
    }, {
        key: "_raiseOnProgress",
        value: function(progressInfo, state) {
            var commonValue = 0 !== progressInfo.totalBytesCount ? progressInfo.uploadedBytesCount / progressInfo.totalBytesCount : 1;
            var fileValue = 0 !== state.file.size ? state.uploadedBytesCount / state.file.size : 1;
            var args = {
                fileIndex: state.fileIndex,
                commonValue: commonValue,
                fileValue: fileValue
            };
            this._onProgressHandler(args)
        }
    }, {
        key: "_createUploadingState",
        value: function(file, fileIndex) {
            var chunkCount = Math.ceil(file.size / this._controller.chunkSize);
            return {
                file: file,
                fileIndex: fileIndex,
                uploadedBytesCount: 0,
                uploadedChunksCount: 0,
                totalChunkCount: chunkCount,
                customData: {}
            }
        }
    }, {
        key: "_createResultInfo",
        value: function(fileName, success, canceled, error) {
            return {
                fileName: fileName,
                success: success || false,
                canceled: canceled || false,
                error: error || null
            }
        }
    }]);
    return FileManagerUploadSession
}();
module.exports = FileManagerFileUploader;
